#include <cstdlib>   // defines atoi, atof
#include <iostream>  // defines std::cout, std::endl, ...
#include <cmath>     // defines std::pow, sin
#include <array>     // defines fixed-size STL container std::array for representing vectors in R^2

#include "explicit_euler_scheme.hh"


// Simulationsprogramm
int main (int argc, char* argv[])
{
  if (argc != 2 && argc != 3)
  {
    std::cout
      << "Bitte den Exponenten i für die Schrittweite als Parameter angeben!"
      << std::endl
      << "Im optionalen zweiten Parameter kann darüber hinaus der Anfangswert "
      << "phi_0 festgelegt werden (standardmäßig wird phi_0 = 0.5 gewählt)."
      << std::endl;
    return 1;
  }

  // Schrittweiteneinstellung
  const int i = atoi(argv[1]);

  // Konstanten
  const double g = 9.80665;
  const double l = 1.0;
  const double T = 12.0;
  const double dt = std::pow(2.0,-1*i);

  // Anfangswerte
  const double phi_0 = (argc == 3) ? atof(argv[2]) : 0.5;
  const double v_0 = 0.0;

  // Datentyp fuer Vektoren des R^2 festlegen
  typedef std::array<double,2> Vector;

  // Instanz der rechten Seite erstellen, z.B. mit Hilfe von Lambda funktionen

  // Instanz des expliziten Eulerverfahrens erstellen
  ExplicitEulerScheme<Vector> explicitEuler(f);

  // Initialisierung
  double t_k = 0;
  Vector y_k;
  y_k[0] = phi_0;
  y_k[1] = v_0;

  // Zeitschritte
  while (t_k + dt <= T)
  {
    // Daten ausgeben
    std::cout << t_k << " " << y_k[0] << std::endl;

    // Eulerschritt durchfuehren
    Vector y_new;
    explicitEuler.apply(t_k,dt,y_k,y_new);

    // Naechsten Eulerschritt vorbereiten
    t_k += dt;
    y_k = y_new;
  }

  // Daten fuer Endzeitpunkt ausgeben
  std::cout << t_k << " " << y_k[0] << std::endl;

  return 0;
}
